// Copyright (C) 2013 - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

// Reference
// http://en.wikipedia.org/wiki/Chebyshev_polynomials

///////////////////////////////////
//
// Evaluate Chebyshev Polynomials of the first kind
//

function y=ChebyshevPoly(n)
    // Create a Chebyshev polynomial of degree n
    if (n==0) then
        y=poly(1,"x","coeff")
    elseif (n==1) then
        y=poly([0 1],"x","coeff")
    else
        polyx=poly([0 1],"x","coeff")
        // y(n-2)
        yn2=poly(1,"x","coeff")
        // y(n-1)
        yn1=polyx
        for k=2:n
            y=2*polyx*yn1-yn2
            yn2=yn1
            yn1=y
        end
    end
endfunction

for n=0:10
    y=ChebyshevPoly(n);
    disp(y)
end

x=linspace(-1,1,100);
y0=horner(ChebyshevPoly(0),x);
y1=horner(ChebyshevPoly(1),x);
y2=horner(ChebyshevPoly(2),x);
y3=horner(ChebyshevPoly(3),x);
h=scf();
plot(x,y0,"r-")
plot(x,y1,"g-")
plot(x,y2,"b-")
plot(x,y3,"c:")
xtitle("Chebyshev Polynomials","x","Tn(x)")
legend(["n=0","n=1","n=2","n=3"]);
h.children.data_bounds(1:2,2)=[-1.05;1.05];
h.children.children(4).children.thickness=3;
h.children.children(5).children.thickness=2;
h.children.children(5).children.line_style=5;

///////////////////////////////////
//
// Check that int Tn(x)^2 w(x) dx=%pi/2
// for w(x)=1/sqrt(1-x^2)
//

function y=sqCw(x,n)
    P=ChebyshevPoly(n)
    y=(horner(P,x))^2/sqrt(1-x^2)
endfunction

function y=ChebyshevNorm(n)
    // Returns Int Tn(x)^2 w(x) dx
    y=%pi/2
endfunction

for n = 1:5
    [v,err]=intg(-1,1,list(sqCw,n));
    I=ChebyshevNorm(n);
    disp([n,I,v])
end
