// Copyright (C) 2013 - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

// Reference
// http://en.wikipedia.org/wiki/Legendre_polynomials

///////////////////////////////////
//
// Evaluate Laguerre Polynomials
//


function y=LaguerrePoly(n)
    // Create a Laguerre polynomial of degree n
    if (n==0) then
        y=poly(1,"x","coeff")
    elseif (n==1) then
        y=poly([1 -1],"x","coeff")
    else
        polyx=poly([0 1],"x","coeff")
        // y(n-2)
        yn2=poly(1,"x","coeff")
        // y(n-1)
        yn1=1-polyx
        for k=2:n
            y=((2*k-1-polyx)*yn1-(k-1)*yn2)/k
            yn2=yn1
            yn1=y
        end
    end
endfunction

for n=0:10
    y=LaguerrePoly(n);
    disp(factorial(n)*y)
end

x=linspace(0,10,100);
y0=horner(LaguerrePoly(0),x);
y1=horner(LaguerrePoly(1),x);
y2=horner(LaguerrePoly(2),x);
y3=horner(LaguerrePoly(3),x);
h=scf();
plot(x,y0,"r-")
plot(x,y1,"g-")
plot(x,y2,"b-")
plot(x,y3,"c:")
xtitle("Laguerre Polynomials","x","Ln(x)")
legend(["n=0","n=1","n=2","n=3"]);
h.children.data_bounds(1:2,1)=[0;7];
h.children.data_bounds(1:2,2)=[-5;5];
h.children.children(4).children.thickness=3;
h.children.children(5).children.thickness=2;
h.children.children(5).children.line_style=5;

///////////////////////////////////
//
// Check that int Ln(x)^2 w(x) dx=1
// for w(x)=exp(-x)
//

function y=sqLw(x,n)
    P=LaguerrePoly(n)
    y=(horner(P,x))^2*exp(-x)
endfunction

function y=LaguerreNorm(n)
    // Returns Int Ln(x)^2 w(x) dx
    y=1
endfunction

for n = 1:5
    [v,err]=intg(0,100,list(sqLw,n));
    I=LaguerreNorm(n);
    disp([n,I,v])
end
