// Copyright (C) 2013 - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

// Reference
// http://en.wikipedia.org/wiki/Legendre_polynomials

///////////////////////////////////
//
// Evaluate Legendre Polynomials
//


function y=Legendre(x,n)
    // Evaluate a Legendre polynomial of degree n at x
    if (n==0) then
        y=1
    elseif (n==1) then
        y=x
    else
        // y(n-2)
        yn2=1
        // y(n-1)
        yn1=x
        for k=2:n
            y=((2*k-1)*x.*yn1-(k-1)*yn2)/k
            yn2=yn1
            yn1=y
        end
    end
endfunction

x=linspace(-1,1,100);
y0=Legendre(x,0);
y1=Legendre(x,1);
y2=Legendre(x,2);
y3=Legendre(x,3);
h=scf();
plot(x,y0,"r-")
plot(x,y1,"g-")
plot(x,y2,"b-")
plot(x,y3,"c:")
xtitle("Legendre Polynomials","x","Pn(x)")
legend(["n=0","n=1","n=2","n=3"]);
h.children.data_bounds(1:2,2)=[-1.05;1.05];
h.children.children(4).children.thickness=3;
h.children.children(5).children.thickness=2;
h.children.children(5).children.line_style=5;

function y=LegendrePoly(n)
    // Create a Legendre polynomial of degree n
    if (n==0) then
        y=poly(1,"x","coeff")
    elseif (n==1) then
        y=poly([0 1],"x","coeff")
    else
        polyx=poly([0 1],"x","coeff")
        // y(n-2)
        yn2=poly(1,"x","coeff")
        // y(n-1)
        yn1=polyx
        for k=2:n
            y=((2*k-1)*polyx*yn1-(k-1)*yn2)/k
            yn2=yn1
            yn1=y
        end
    end
endfunction

for n=0:10
    y=LegendrePoly(n);
    disp(y)
end

///////////////////////////////////
//
// Check that int Pn(x)^2 w(x) dx=2/(2n+1)
// for w(x)=1
//

function y=sqPw(x,n)
    y=Legendre(x,n)^2
endfunction

function y=LegendreNorm(n)
    // Returns Int Pn(x)^2 w(x) dx
    y=2/(2*n+1)
endfunction

for n = 1:5
    [v,err]=intg(-1,1,list(sqPw,n));
    I=LegendreNorm(n);
    disp([n,I,v])
end
